unit LZSS_OptParser;

interface

uses Windows, Classes, SysUtils, Math;

Const
  TLZ_LITERAL       = 1;
  TLZ_MATCH         = 0;
  TLZ_MINMATCHLEN   = 2;
  TLZ_WINBITS       = 10;
  TLZ_MATCHBITS     = 4;
  TLZ_WINLEN        = 1023;
  TLZ_MATCHLEN      = 17;
  M                 = 128;
type
  TLZ = Class
  public
    function Compress(const Source; var Dest; Size: ULONG): ULONG;
    function Decompress(const Source; var Dest; Size: PULONG): ULONG;
  private

    DataStreamPos: ULONG;
    CompressedStreamPos: ULONG;
    Data:array of UCHAR;
    CompressedData:array of UCHAR;

    DataSize: ULONG;
    CompressedSize: ULONG;
    CompressedLong: ULONG;
    CompressedBitsUsed: Integer;

    //Functions

    function CompressLoop(): ULONG;
    procedure FindMatches(Pos: ULONG);
    procedure CompressedStreamReadBits(Value: PUINT; NumBits: UINT);
    procedure CompressedStreamWriteBits(Value : UINT; NumBits: UINT);
    procedure CompressedStreamWriteBitsFlush();
end;

var
	Val, Size, Link, Pos, Len: array[0..M] of Integer;
	MatchLen, MatchPos: Integer;


implementation


{ TLZ }

function TLZ.Compress(const Source; var Dest; Size: ULONG): ULONG;
begin
  //Set up initial values
  DataStreamPos:=        0;
  CompressedStreamPos:=  0;
  CompressedLong:=       0;
  CompressedBitsUsed:=   0;
  CompressedData:=       @Dest;
  Data:=                 @Source;
  DataSize :=            Size;
  CompressedSize:=       0;

  CompressLoop;

  CompressedStreamWriteBitsFlush;
  Result := CompressedSize;
end;

procedure TLZ.CompressedStreamReadBits(Value: PUINT; NumBits: UINT);
var
  Temp: ULONG;
begin
  CompressedLong:= CompressedLong and $FFFF;
  while NumBits > 0 do
  begin
    dec(NumBits);
    if CompressedBitsUsed = 0 then
    begin
      Temp:= CompressedData[CompressedStreamPos];
      inc(CompressedStreamPos);
      CompressedLong:= CompressedLong or (Temp shl 8);
      Temp:= CompressedData[CompressedStreamPos];
      inc(CompressedStreamPos);
      CompressedLong:= CompressedLong or Temp;
      CompressedBitsUsed:= 16;
    end;
    CompressedLong:= CompressedLong shl 1;
    dec(CompressedBitsUsed);
  end;
  Value^:= CompressedLong shr 16;
end;

procedure TLZ.CompressedStreamWriteBits(Value, NumBits: UINT);
begin
  while NumBits > 0 do
  begin
    dec(NumBits);
    CompressedLong := CompressedLong shl 1;
    CompressedLong:= CompressedLong or ((Value shr NumBits) and 1);
    inc(CompressedBitsUsed);

    if(CompressedBitsUsed = 32) then
    begin
      CompressedData[CompressedStreamPos]:= CompressedLong shr 24;
      CompressedData[CompressedStreamPos+1]:= CompressedLong shr 16;
      inc(CompressedStreamPos,2);
      inc(CompressedSize,2);
      dec(CompressedBitsUsed,16);
    end;
  end;  //end while
end;

procedure TLZ.CompressedStreamWriteBitsFlush;
begin
  CompressedLong:= CompressedLong shl (32 - CompressedBitsUsed);
  case CompressedBitsUsed of
  1..8:
    begin
      CompressedData[CompressedStreamPos]:= CompressedLong shr 24;
      inc(CompressedSize);
      inc(CompressedStreamPos);
    end;
  9..16:
    begin
      CompressedData[CompressedStreamPos]:= CompressedLong shr 24;
      CompressedData[CompressedStreamPos+1]:= CompressedLong shr 16;
      inc(CompressedStreamPos, 2);
      inc(CompressedSize, 2);
    end;
  17..24:
    begin
      CompressedData[CompressedStreamPos]:= CompressedLong shr 24;
      CompressedData[CompressedStreamPos+1]:= CompressedLong shr 16;
      CompressedData[CompressedStreamPos+2]:= CompressedLong shr 8;
      inc(CompressedStreamPos, 3);
      inc(CompressedSize, 3);
    end;
  25..32:
    begin
      CompressedData[CompressedStreamPos]:= CompressedLong shr 24;
      CompressedData[CompressedStreamPos+1]:= CompressedLong shr 16;
      CompressedData[CompressedStreamPos+2]:= CompressedLong shr 8;
      CompressedData[CompressedStreamPos+3]:= CompressedLong;
      inc(CompressedStreamPos, 4);
      inc(CompressedSize, 4);
    end;
  end;
      CompressedBitsUsed:= 0;

end;

function TLZ.CompressLoop: ULONG;
var
	a, b, c, d, e, cc: Integer;
	k, n, i: Integer;
begin
  CompressedStreamWriteBits(Data[0], 8);
  Inc(DataStreamPos);
  while DataStreamPos < DataSize do
  begin
    FindMatches(DataStreamPos);
    a:= 0;
    b:= 1;
    Link[1]:= 0;
    Size[0]:= 0; Size[1]:= TLZ_MINMATCHLEN;// r:= P/C  P -       +  ,  -      
    Val[0]:= DataStreamPos; Val[1]:= DataStreamPos + MatchLen;
    Len[1]:= MatchLen;
    Pos[1]:= MatchPos;
    k:= 1;
    while true do
    begin
      FindMatches(Val[a] + k);
      d:= Val[a] + k + MatchLen - Val[b];
      c:= Size[a] + k + TLZ_MINMATCHLEN;
      cc:= Size[b] + Min(d,TLZ_MINMATCHLEN);
    //2.
      if not (c >= cc) then
      begin
        Inc(b);
        Link[b]:= a;
        Size[b]:= Size[a] + k +TLZ_MINMATCHLEN;
        Val[b]:= Val[a] + k + MatchLen;
        Len[b]:= MatchLen;
        Pos[b]:= MatchPos;
      end;
    // 4.
      Inc(k);
    //5.
      if b = Pred(M) then
      begin
        e:= b;
        Break;
      end;
      if k < (Size[a+1] - Size[a]) then
        continue;
    //6.
      k:= 0;
      Inc(a);
    //7.
      if a < b then
        continue;
    //8.
      e:= b;
      break;
    end;

    n:= -1;
    repeat
    begin
      k:= Link[e];
      Link[e]:= n;
      n:= e;
      e:= k;
    end until not (e > 0);

    repeat
    begin
      n:= Val[n] - Val[e] - Len[n];
      k:= 0;
      while k < n do
      begin
        CompressedStreamWriteBits(TLZ_LITERAL, 1);
        CompressedStreamWriteBits(Data[k + Val[e]], 8);
        Inc(k);
      end;
        e:= n;
        CompressedStreamWriteBits(TLZ_MATCH, 1);
        CompressedStreamWriteBits(Pos[e], TLZ_WINBITS);
        CompressedStreamWriteBits(Len[e], TLZ_MATCHBITS);
        n:= Link[e];
    end until not Link[e] >= 0;
    DataStreamPos:= Val[e];
  end;  // end while
  CompressedStreamWriteBits(TLZ_MATCH, 1);
  CompressedStreamWriteBits(0,TLZ_WINBITS); // Flag  
end;  // Compress Loop

procedure TLZ.FindMatches(Pos: ULONG);
var
  WPos: ULONG;
  TempLen: ULONG;
  MaxWPos: ULONG;
begin
  MatchLen:= TLZ_MINMATCHLEN - 1;
  if Pos < TLZ_WINLEN then
    WPos := 0
  else
    WPos:= Pos - TLZ_WINLEN;
  MaxWPos:= Pos;
  while (WPos < MaxWPos) do
  begin
    TempLen:= 0;
    while((Data[WPos + TempLen] = Data[Pos + TempLen]) and ((Pos + TempLen) < DataSize) and (TempLen < TLZ_MATCHLEN)) do
    begin
      inc(TempLen);
    end;  // end while
    if (TempLen > MatchLen) and ((WPos and TLZ_WINLEN) <> 0) then
    begin
      MatchLen:= TempLen;
      MatchPos:= WPos;
    end;
    Inc(WPos);
  end;
end;

function TLZ.Decompress(const Source; var Dest; Size: PULONG): ULONG;
var
  Temp: UINT;
  Offset, Len: UINT;
  Buffer: array[0..TLZ_WINLEN] of Byte;
begin
  DataStreamPos:= 0;
  CompressedStreamPos:= 0;
  CompressedLong:= 0;
  CompressedBitsUsed:= 0;
  CompressedData:= @Source;
  Data:= @Dest;

  while true do
  begin
    CompressedStreamReadBits(@Temp,1);

    if Temp = TLZ_LITERAL then
    begin
      CompressedStreamReadBits(@Temp, 8);
      Data[DataStreamPos]:= Temp;
      Buffer[DataStreamPos and TLZ_WINLEN]:= Temp;
      inc(DataStreamPos);
    end
    else
    begin
      CompressedStreamReadBits(@Offset,TLZ_WINBITS);
      if Offset = 0 then break;
      CompressedStreamReadBits(@Len, TLZ_MATCHBITS);
      Len:= Len + TLZ_MINMATCHLEN;
      while Len > 0 do
      begin
        dec(Len);
        Data[DataStreamPos]:= Buffer[Offset and TLZ_WINLEN];
        Buffer[DataStreamPos and TLZ_WINLEN]:= Data[DataStreamPos];
        inc(DataStreamPos);
        inc(Offset);
      end;
    end;
  end;
  Size^:= CompressedStreamPos;
  Result:= DataStreamPos;
end;

end.
