unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, LZSS_OptParser, ExtCtrls, XPMan;

type
  TForm1 = class(TForm)
    bCompress: TButton;
    bDecompress: TButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    Address: TEdit;
    Label1: TLabel;
    XPManifest1: TXPManifest;
    Box: TGroupBox;
    RadioButton1: TRadioButton;
    RadioButton2: TRadioButton;
    RadioButton3: TRadioButton;
    RadioButton4: TRadioButton;
    PSize: TEdit;
    Label2: TLabel;
    procedure bCompressClick(Sender: TObject);
    procedure bDecompressClick(Sender: TObject);
    procedure RadioButton1Click(Sender: TObject);
    procedure RadioButton2Click(Sender: TObject);
    procedure RadioButton4Click(Sender: TObject);
    procedure RadioButton3Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

type
  TTilesHeader = record
    Size: Word;
  end;
  TMapHeader = record
    MapWidth: Word;
    MapHeight: Word;
  end;

const
  MapMode   = 0;
  TilesMode = 1;
  WithoutHeaderMode = 2;
var
  Form1: TForm1;
  MapHeader: TMapHeader;
  TilesHeader: TTilesHeader;
  Mode: Byte = 0;
implementation

{$R *.dfm}

procedure TForm1.bCompressClick(Sender: TObject);
var
  MyFile: TFileStream;
  SaveFile: TFileStream;
  Buffer, OutBuffer: array of Byte;
  Size: LongWord;
  CompSize: Integer;
  LZ: TLZ;
begin
  if OpenDialog1.Execute then
  begin
    MyFile:= TFileStream.Create(OpenDialog1.FileName, fmOpenRead);
    Size:= MyFile.Size;
    case Mode of
      MapMode:
        begin
          MyFile.ReadBuffer(MapHeader, SizeOf(TMapHeader));
          //MyFile.ReadBuffer(MapHeader.MapHeight, 2);
          dec(Size, 4);
        end;
      TilesMode:
        begin
          TilesHeader.Size:= Swap(Size);
        end;
    end;
    SetLength(Buffer, Size);
    MyFile.ReadBuffer(Buffer[0], Length(Buffer));
    MyFile.Free;
    LZ:= TLZ.Create;
    SetLength(OutBuffer, Size);
    CompSize:= LZ.Compress(Buffer[0], OutBuffer[0], Size);
    LZ.Free;
    if SaveDialog1.Execute then
    begin
      SaveFile:= TFileStream.Create(SaveDialog1.FileName, fmCreate);
      case Mode of
        MapMode: SaveFile.WriteBuffer(MapHeader, SizeOf(TMapHeader));
        TilesMode: SaveFile.WriteBuffer(TilesHeader, SizeOf(TTilesHeader));
      end;
      SaveFile.WriteBuffer(OutBuffer[0], CompSize);
      SaveFile.Free;
    end;
  end;
end;

procedure TForm1.bDecompressClick(Sender: TObject);
var
  MyFile: TFileStream;
  UncompFile: TFileStream;
  Size, UnCompSize: LongWord;
  Buffer, OutBuffer: array of Byte;
  LZ: TLZ;
  hAddress: LongWord;
  CompressedSize: LongWord;
begin
  if Opendialog1.Execute then
  begin
    MyFile:= TFileStream.Create(OpenDialog1.FileName, fmOpenRead);
    hAddress:= StrToInt(Address.Text);
    MyFile.Position:= hAddress;
    Size:= MyFile.Size;
    if Size < hAddress then
    begin
      MessageBox(0, 'Invalid Address', 'Warning!', mb_Ok or mb_IconWarning);
      MyFile.Free;
      exit;
    end;
    case Mode of
      MapMode:
        begin
          MyFile.ReadBuffer(MapHeader, SizeOf(TMapHeader));
          if (Size - hAddress) > $FFFF then
            Size:= $FFFF
          else
            Size:= Size - hAddress - SizeOf(TMapHeader);
        end;
      TilesMode:
        begin
          MyFile.ReadBuffer(TilesHeader, SizeOf(TTilesHeader));
          if (Size - hAddress) > $FFFF then
            Size:= $FFFF
          else
            Size:= Size - hAddress - SizeOf(TTilesHeader);
        end;
      WithoutHeaderMode:
        begin
          if (Size - hAddress) > $FFFF then
            Size:= $FFFF
          else
            Size:= Size - hAddress;
        end;
    end;

    SetLength(Buffer, Size);
    MyFile.ReadBuffer(Buffer[0], Length(Buffer));
    MyFile.Free;
    LZ:= TLZ.Create;
    SetLength(OutBuffer, $FFFF);
    UnCompSize:= LZ.Decompress(Buffer[0], OutBuffer[0], @CompressedSize);
    LZ.Free;
    if SaveDialog1.Execute then
    begin
      UnCompFile:= TFileStream.Create(SaveDialog1.FileName, fmCreate);
      case Mode of
        MapMode:
          begin
           PSize.Text:= IntToStr(CompressedSize + SizeOf(TMapHeader));
           UnCompFile.WriteBuffer(MapHeader, SizeOf(TMapHeader));
          end;
        TilesMode:
          begin
            PSize.Text:= IntToStr(CompressedSize + SizeOf(TTilesHeader));
          end;
        WithoutHeaderMode:
          begin
            PSize.Text:= IntToStr(CompressedSize);
          end;
      end;
      UnCompFile.WriteBuffer(OutBuffer[0], UnCompSize);
      UnCompFile.Free;
    end;
  end;
end;
procedure TForm1.RadioButton1Click(Sender: TObject);
begin
  bCompress.Caption:= 'Compress Map';
  bDecompress.Caption:= 'Decompress Map';
  Mode:= MapMode;
end;

procedure TForm1.RadioButton2Click(Sender: TObject);
begin
  bCompress.Caption:= 'Compress Tiles';
  bDecompress.Caption:= 'Decompress Tiles';
  Mode:= TilesMode;
end;

procedure TForm1.RadioButton4Click(Sender: TObject);
begin
  bCompress.Caption:= 'Compress Tiles';
  bDecompress.Caption:= 'Decompress Tiles';
  Mode:= WithoutHeaderMode;
end;

procedure TForm1.RadioButton3Click(Sender: TObject);
begin
  bCompress.Caption:= 'Compress Map';
  bDecompress.Caption:= 'Decompress Map';
  Mode:= WithoutHeaderMode;
end;

end.
